#include <zephyr/kernel.h>
#include <zephyr/sys/printk.h>
#include <zephyr/device.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/devicetree.h>
#include <stdio.h>
#include "libraries/SPI_IO_nRF.h"
#include "libraries/ads131m0x.h"
#include "libraries/ads131_reg_defs.h"
#include "libraries/max20303.h"

#define SPIOP_ADS131 (SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_MODE_CPHA)

#define ADS_NODE DT_NODELABEL(ads131m02)
#define SPI_NODE DT_NODELABEL(spi3)

static const struct gpio_dt_spec     cs_gpio  = GPIO_DT_SPEC_GET(SPI_NODE, cs_gpios);
static const struct spi_cs_control   cs_ctrl  = {
    .gpio       = cs_gpio,
    .delay      = 0,  
};

static const struct spi_dt_spec spi_dev = SPI_DT_SPEC_GET(ADS_NODE,
    SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_MODE_CPHA, 0);

static const struct spi_config spi_cfg = {
    .frequency = DT_PROP(ADS_NODE, spi_max_frequency),
    .operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_MODE_CPHA,
    .slave     = DT_REG_ADDR(ADS_NODE),
    .cs        = &cs_ctrl,
};

const struct gpio_dt_spec drdy_gpio = GPIO_DT_SPEC_GET(ADS_NODE, drdy_gpios);

// static struct gpio_callback drdy_cb;

// volatile bool interrupt_flag = false;
// volatile bool busy = false;

// #define NUM_SAMPLES 1024
// int sample_index = 0;

// uint32_t start_time = 0;
// uint32_t end_time = 0;

// void drdy_isr_handler(const struct device *port, struct gpio_callback *cb, uint32_t pins)
// {
//     ARG_UNUSED(port);
//     ARG_UNUSED(cb);
//     ARG_UNUSED(pins);
//     interrupt_flag = true;
// }

int main(void)
{
        if (!I2C_Ready()) {
                printk("I2C could not be configured\n");
                return 0;
        }

        while (MAX20303_Stay_On() < 0) {
                printk("MAX20303 unable to be put into STAY_ON mode...\n");
                k_msleep(1000);
        }

        while (MAX20303_Set_Buck2_Voltage(3)) {
                printk("MAX20303 unable to be put into STAY_ON mode...\n");
                k_msleep(1000);
        }

        if (!spi_is_ready_dt(&spi_dev)) {
                printk("SPI not ready\n");
                return 0;
        }

        int ret;

        uint8_t null_command[3] = {0x00};

        struct spi_buf empty_spi_buf = {.buf = null_command, .len = 3};
        struct spi_buf_set empty_spi_buf_set 	= {.buffers = &empty_spi_buf, .count = 1};

        uint8_t tx_buf[3] = {0xA0, 0x00, 0x00};
        uint8_t rx_buf[3] = {0x00};

        struct spi_buf tx_spi_buf = {.buf = tx_buf, .len = 3};
        struct spi_buf_set tx_spi_buf_set 	= {.buffers = &tx_spi_buf, .count = 1};

        struct spi_buf rx_spi_bufs 		= {.buf = rx_buf, .len = 3};
        struct spi_buf_set rx_spi_buf_set	= {.buffers = &rx_spi_bufs, .count = 1};

        printk("Alive here\n");

        while (true){
                printk("TEST!\n");
                ret = spi_transceive(spi_dev.bus, &spi_cfg, &tx_spi_buf_set, &rx_spi_buf_set);
                printk("TEST 2!\n"); // Program seems to stop running here
                if (ret < 0) {
                        printk("spi_transeive() failed, err: %d\n", ret);
                }

                ret = spi_transceive(spi_dev.bus, &spi_cfg, &empty_spi_buf_set, &rx_spi_buf_set);

                if (ret < 0) {
                        printk("spi_transeive() failed, err: %d\n", ret);
                }
                printk("%x\n", rx_buf[0]);
                k_msleep(1000);
        }
        // uint16_t device_id = 0;
        // while ((uint8_t)(device_id >> 8) != 0x24 && (uint8_t)(device_id >> 8) != 0x22) {
        //         if(ADS131_GetID(&spi_dev, &device_id) != ADS131M0x_OK) {
        //                 printk("ADS131 - Error Code Thrown: GetID()");
        //         }

        //         printk("ADS131 Device ID: %x\n", (uint8_t)(device_id >> 8));
        //         k_msleep(1000);
        // }

        // ADS131M0x_CLOCK_OSR_t clock_osr = 0;

        // while (clock_osr != ADS131M0x_CLOCK_OSR_8192) {
        //         if (ADS131_Clock_SetOSR(&spi_dev, ADS131M0x_CLOCK_OSR_8192) != ADS131M0x_OK) {
        //                 printk("ADS131 - Error Code Thrown: SetOSR()");
        //         }
        //         k_msleep(10);
        //         if (ADS131_Clock_GetOSR(&spi_dev, &clock_osr) != ADS131M0x_OK) {
        //                 printk("ADS131 - Error Code Thrown: GetOSR()");
        //         }
        //         printk("%x\n", clock_osr);
        //         k_msleep(1000);
        // }

        // while (true) {
        //         if(ADS131_GetID(&spi_dev, &device_id) != ADS131M0x_OK) {
        //                 printk("ADS131 - Error Code Thrown: GetID()");
        //         }
        //         printk("ADS131 Device ID: %x\n", (uint8_t)(device_id >> 8));
        //         k_msleep(1000);
        // }
        return 0;
}
